scriptname _DE_WoodHarvestScript extends ReferenceAlias

import debug
import utility

int property myHarvest auto
Actor property pPlayer auto
weapon property Axe01 auto
weapon property dunHaltedStreamPoachersAxe auto
message property _DE_WoodHarvest auto
message property _DE_WoodHarvestError1 auto
message property _DE_WoodHarvestError2 auto
message property _DE_WoodHarvestError3 auto
message property _DE_WoodHarvestSuccess auto
formlist property _DE_Trees auto
furniture property _DE_WoodPilePlayer auto
MiscObject property Firewood01 auto
objectreference property _DE_XMarker_WoodHarvestEndRef2 auto			;I may not need this
objectreference property _DE_Anchor auto
package property _DE_PlayerHarvestWoodPathing2 auto
int property myFirewoodCount auto
objectreference property myWoodFurnStartRef auto
objectreference property myWoodFurnEndRef auto
objectreference property myWoodContEndRef auto
ImageSpaceModifier Property _DE_FadeDown auto
ImageSpaceModifier Property _DE_FadeUp auto
ImageSpaceModifier Property _DE_Black auto
globalvariable property TimeScale auto
globalvariable property GameHour auto
globalvariable property _DE_Setting_WoodCinematic auto
globalvariable property _DE_ExposurePoints auto
container property _DE_FirewoodContainer auto
ReferenceAlias property _DE_WoodHarvestingAlias auto

float property pfThreadLastUpdateTime auto

event OnObjectEquipped(Form akBaseObject, ObjectReference akReference)
	if akBaseObject as Weapon
		if akBaseObject == Axe01 ||	akBaseObject == dunHaltedStreamPoachersAxe
			if !Game.GetPlayer().IsInCombat()
				if !Game.GetPlayer().IsInInterior()
					if Game.FindClosestReferenceOfAnyTypeInListFromRef(_DE_Trees, Game.GetPlayer(), 3000.0) != none
						WoodHarvestMenu()
					else
						_DE_WoodHarvestError3.Show()
					endif
				else
					_DE_WoodHarvestError1.Show()
				endif
			else
				_DE_WoodHarvestError2.Show()
			endif
		endif
	endif
endEvent

function WoodHarvestMenu()
	
	;Don't allow this to be used in interiors, or in combat.
	
	int ibutton = _DE_WoodHarvest.Show()
	if ibutton == 0
		if _DE_Setting_WoodCinematic.GetValueInt() == 2
			HarvestWood()
		else
			HarvestWoodNoPackage()
		endif
	elseif ibutton == 1
		;exit
	endif
	
endfunction

function HarvestWoodNoPackage()
	
	;A simpler wood harvesting system for those that might have trouble with the more complex version
	
	pPlayer = Game.GetPlayer()
	Game.DisablePlayerControls()
		
	myFirewoodCount = pPlayer.GetItemCount(Firewood01)
	
	;If the player has any firewood in his inventory now, get rid of it (and add it back later)
	;This ensures proper behavior when interacting with the furniture objects
	if myFirewoodCount > 0
		pPlayer.RemoveItem(Firewood01, myFirewoodCount, true)		;silently
	endif
	
	;fade to black
	_DE_FadeDown.Apply()
	Wait(1)
	_DE_FadeDown.PopTo(_DE_Black)
	
	;Advance time by 1 hour
	if GameHour.GetValue() <= 22.9
		GameHour.SetValue(GameHour.GetValue() + 1.0)
	else
		GameHour.SetValue(0.0)
	endif
	
	;Trigonometry time! We need to position the furniture object such that the furniture marker is facing the player when they approach it.
	;We also need to move the player 300 units away from the direction they is facing (diagonally), but we can only set XPos and YPos.
	int myQuad = 1
	float mySideC = 100.0	;Hypotenuse (distance to move the player)
	float myAngleA = pPlayer.GetAngleZ()
	
	float myAngleC = 90.0
	
	;Clamp to an angle less than 90 degrees.
	if myAngleA > 90.0 && myAngleA <= 180.0
		myAngleA -= 90.0
		myQuad = 2
	elseif myAngleA > 180.0 && myAngleA <= 270.0
		myAngleA -= 180.0
		myQuad = 3
	elseif myAngleA > 270.0
		myAngleA -= 270.0
		myQuad = 4
	endif
	
	;Determine the length of the legs of the triangle for moving the player.
	float mySideA = math.sin(myAngleA) * mySideC				;Y offset
	float mySideB = math.cos(myAngleA) * mySideC				;X offset
	
	;Figure out which direction the offset should travel based on the quadrant. We use adjacent diagonal quadrant because we are moving BACKWARDS.
	float myXOffset
	float myYOffset
	
	if myQuad == 1
		;Move into the 3rd quadrant
		myXOffset = mySideA * -1.0
		myYOffset = mySideB * -1.0
	elseif myQuad == 2
		;Move into the 4th quadrant
		myXOffset = mySideA * -1.0
		myYOffset = mySideB
	elseif myQuad == 3
		;Move into the 1st quadrant
		myXOffset = mySideA
		myYOffset = mySideB
	elseif myQuad == 4
		;Move into the 2nd quadrant
		myXOffset = mySideA
		myYOffset = mySideB * -1.0
	endif
	
	myWoodContEndRef = Game.FindClosestReferenceOfTypeFromRef(_DE_FirewoodContainer, pPlayer, 2000.0)
	
	;Put the wood pile container near the player, if not within 2000 units of the player already
	if myWoodContEndRef != None
		if !myWoodContEndRef.IsDisabled()
			MoveOldFirewoodPileNoCinematic(myXOffset, myYOffset)
		else
			MoveNewFirewoodPileNoCinematic(myXOffset, myYOffset)
		endif
	else
		MoveNewFirewoodPileNoCinematic(myXOffset, myYOffset)
	endif
	
	Game.EnablePlayerControls()
	
	wait(2)
	_DE_Black.PopTo(_DE_FadeUp)
	
	;Give the player back his firewood
	if myFirewoodCount > 0
		pPlayer.AddItem(Firewood01, myFirewoodCount, true)		;silently
	endif
	
	;Give the player the new firewood.
	myHarvest = RandomInt(3, 7)
	_DE_WoodHarvestSuccess.Show(myHarvest)
	myWoodContEndRef.AddItem(Firewood01, myHarvest)
	
endFunction

function MoveNewFirewoodPileNoCinematic(float myXOffset, float myYOffset)
	;Create a new wood pile container and set up the player.
	_DE_XMarker_WoodHarvestEndRef2.MoveTo(pPlayer)
	_DE_XMarker_WoodHarvestEndRef2.SetAngle(0.0, 0.0, pPlayer.GetAngleZ())
	myWoodContEndRef = _DE_XMarker_WoodHarvestEndRef2.PlaceAtMe(_DE_FirewoodContainer)
	myWoodContEndRef.SetAngle(0.0, 0.0, pPlayer.GetAngleZ() + 180)
	
	;Move the player 300 units away from current heading.
	pPlayer.MoveTo(pPlayer, myXOffset, myYOffset, 0.0)
endfunction

function MoveOldFirewoodPileNoCinematic(float myXOffset, float myYOffset)
	;There is already a wood pile container located nearby, move the player there and use it instead.
	pPlayer.MoveTo(myWoodContEndRef)
	pPlayer.SetAngle(0.0, 0.0, myWoodContEndRef.GetAngleZ() + 180.0)
	
	;Move the player 100 units away from current heading.
	pPlayer.MoveTo(pPlayer, myXOffset, myYOffset, 0.0)
endFunction


event OnUpdate()
	;The package may not have ended; teleport the player back to where he started and use the furniture, if things take too long
	
	float fThreadTimeDelta = GetCurrentGameTime() - pfThreadLastUpdateTime			;(difference in game-time days)
	float fThreadTimeDeltaSec = (fThreadTimeDelta * 86400)/TimeScale.GetValueInt()		;(difference in real seconds)
	float fWaitTime = (6.0 - fThreadTimeDeltaSec)
	
	if fThreadTimeDeltaSec < 6.0
		RegisterForSingleUpdate(fWaitTime)
		;notification("[Frostfall]Waiting for another " + fWaitTime + " secs...delta " + fThreadTimeDelta)
		;notification("[Frostfall] Current Time: " + GetCurrentGameTime() + ", Last Update: " + pfThreadLastUpdateTime)
	else
		if pPlayer.GetCurrentPackage() == _DE_PlayerHarvestWoodPathing2
			;notification("[Frostfall]Package got stuck, resolving...")
			HarvestWoodEnd()
		else
			;notification("[Frostfall]Package resolved successfully.")
		endif
	endif
	
endEvent
	

event OnPackageEnd(Package akOldPackage)
	if akOldPackage == _DE_PlayerHarvestWoodPathing2
		;notification("I just finished the package")
		HarvestWoodEnd()
	endif
endEvent

event OnAnimationEvent(ObjectReference akSource, string asEventName)
	if asEventName == "AddToInventory"
		;messagebox(akSource)
		;notification("I finished picking up lumber")
		if Game.FindClosestReferenceOfTypeFromRef(_DE_WoodPilePlayer, Game.GetPlayer(), 50.0) != none
			HarvestWoodWalking()
		endif
	elseif asEventName == "RemoveFromInventory"
		if Game.FindClosestReferenceOfTypeFromRef(_DE_WoodPilePlayer, Game.GetPlayer(), 50.0) != none
		;messagebox(akSource)
		;notification("I finished unloading lumber")
			myWoodFurnEndRef.Disable()
			myWoodFurnEndRef.Delete()
		
			;Empty property values (may help with object deletion)
			myWoodContEndRef = none
			myWoodFurnStartRef = none
			myWoodFurnEndRef = none
		
			_DE_WoodHarvestingAlias.Clear()
			;messagebox("Alias is " + _DE_WoodHarvestingAlias)
			UnregisterForAllEvents()
		endif
	endif
endEvent



function HarvestWood()
	
	pPlayer = Game.GetPlayer()
	
	RegisterForAnimationEvent(pPlayer, "AddToInventory")
	RegisterForAnimationEvent(pPlayer, "RemoveFromInventory")
	
	Game.DisablePlayerControls()
		
	myFirewoodCount = pPlayer.GetItemCount(Firewood01)
	
	;If the player has any firewood in his inventory now, get rid of it (and add it back later)
	;This ensures proper behavior when interacting with the furniture objects
	if myFirewoodCount > 0
		pPlayer.RemoveItem(Firewood01, myFirewoodCount, true)		;silently
	endif
	
	;fade to black
	_DE_FadeDown.Apply()
	Wait(1)
	_DE_FadeDown.PopTo(_DE_Black)
	
	Game.ForceThirdPerson()
	
	;Advance time by 1 hour
	if GameHour.GetValue() <= 22.9
		GameHour.SetValue(GameHour.GetValue() + 1.0)
	else
		GameHour.SetValue(0.0)
	endif
	
	;Trigonometry time! We need to position the furniture object such that the furniture marker is facing the player when they approach it.
	;We also need to move the player 300 units away from the direction they is facing (diagonally), but we can only set XPos and YPos.
	int myQuad = 1
	float mySideC = 300.0	;Hypotenuse (distance to move the player)
	float myAngleA = pPlayer.GetAngleZ()
	
	float myAngleC = 90.0
	
	;Clamp to an angle less than 90 degrees.
	if myAngleA > 90.0 && myAngleA <= 180.0
		myAngleA -= 90.0
		myQuad = 2
	elseif myAngleA > 180.0 && myAngleA <= 270.0
		myAngleA -= 180.0
		myQuad = 3
	elseif myAngleA > 270.0
		myAngleA -= 270.0
		myQuad = 4
	endif
	
	;Determine the length of the legs of the triangle for moving the player.
	float mySideA 
	float mySideB
	
	if myQuad == 1 || myQuad == 3
		mySideA = math.sin(myAngleA) * mySideC				;Y offset
		mySideB = math.cos(myAngleA) * mySideC				;X offset
	else
		mySideA = math.cos(myAngleA) * mySideC				;Y offset
		mySideB = math.sin(myAngleA) * mySideC				;X offset
	endif
	
	;Determine the length of the legs of the triangle for placing the furniture (it can't be exactly at the origin, or the player will step on it)
	float myFurnSideC = 65.0	;Hypotenuse (distance to place furniture from origin)
	float myFurnSideA = math.sin(myAngleA) * myFurnSideC				;Y offset
	float myFurnSideB = math.cos(myAngleA) * myFurnSideC				;X offset
	
	;Figure out which direction the offset should travel based on the quadrant. We use adjacent diagonal quadrant because we are moving BACKWARDS.
	float myXOffset
	float myYOffset
	float myFurnXOffset
	float myFurnYOffset
	if myQuad == 1
		;Move into the 3rd quadrant
		myXOffset = mySideA * -1.0
		myYOffset = mySideB * -1.0
		myFurnXOffset = myFurnSideA * -1.0
		myFurnYOffset = myFurnSideB * -1.0
	elseif myQuad == 2
		;Move into the 4th quadrant
		myXOffset = mySideA * -1.0
		myYOffset = mySideB
		myFurnXOffset = myFurnSideA * -1.0
		myFurnYOffset = myFurnSideB
	elseif myQuad == 3
		;Move into the 1st quadrant
		myXOffset = mySideA
		myYOffset = mySideB
		myFurnXOffset = myFurnSideA
		myFurnYOffset = myFurnSideB
	elseif myQuad == 4
		;Move into the 2nd quadrant
		myXOffset = mySideA
		myYOffset = mySideB * -1.0
		myFurnXOffset = myFurnSideA
		myFurnYOffset = myFurnSideB * -1.0
	endif
	
	;notification("Xoffset: " + myXOffset + ", Yoffset: " + myYOffset + ", Quad: " + myQuad)
	
	myWoodContEndRef = Game.FindClosestReferenceOfTypeFromRef(_DE_FirewoodContainer, pPlayer, 2000.0)
	
	;Put the wood pile container near the player, if not within 2000 units of the player already
	if myWoodContEndRef != None
		if !myWoodContEndRef.IsDisabled()
			MoveOldFirewoodPile(myXOffset, myYOffset, myFurnXOffset, myFurnYOffset)
		else
			MoveNewFirewoodPile(myXOffset, myYOffset, myFurnXOffset, myFurnYOffset)
		endif
	else
		MoveNewFirewoodPile(myXOffset, myYOffset, myFurnXOffset, myFurnYOffset)
	endif
			
	;play wood chopping sound.

	;Now that the wood piles have been set up, evaluate AI
	;use furniture, wait until finished, and walk to wood pile
	
	;notification("Picking up wood")
	
	bool bActivated = myWoodFurnStartRef.Activate(pPlayer)					;Make sure there was no problem activating the furniture
	
	if !(bActivated)
		;notification("I couldn't activate the pickup, resolving function...")
		_DE_Black.PopTo(_DE_FadeUp)
		HarvestWoodEnd()
	endif
	
	wait(1)
	if bActivated == false			;Prevents hilarious problem of player moving automatically into river and being carried downstream without ever activating the furniture
		_DE_Black.PopTo(_DE_FadeUp)
		HarvestWoodEnd()
	endif
	
endFunction

function MoveNewFirewoodPile(float myXOffset, float myYOffset, float myFurnXOffset, float myFurnYOffset)
	;Create a new wood pile container and set up the player.
	_DE_XMarker_WoodHarvestEndRef2.MoveTo(pPlayer)
	_DE_XMarker_WoodHarvestEndRef2.SetAngle(0.0, 0.0, pPlayer.GetAngleZ())
	myWoodContEndRef = _DE_XMarker_WoodHarvestEndRef2.PlaceAtMe(_DE_FirewoodContainer)
	myWoodContEndRef.SetAngle(0.0, 0.0, pPlayer.GetAngleZ() + 180)
	
	;Move the marker back and then place the furniture object
	_DE_XMarker_WoodHarvestEndRef2.MoveTo(pPlayer, myFurnXOffSet, myFurnYOffset, -10.0)
	myWoodFurnEndRef = _DE_XMarker_WoodHarvestEndRef2.PlaceAtMe(_DE_WoodPilePlayer)
	myWoodFurnEndRef.SetScale(0.01)
	
	;Move the player 300 units away from current heading.
	pPlayer.MoveTo(pPlayer, myXOffset, myYOffset, 0.0)
	;move this near the player
	myWoodFurnStartRef = pPlayer.PlaceAtMe(_DE_WoodPilePlayer)
endFunction

function MoveOldFirewoodPile(float myXOffset, float myYOffset, float myFurnXOffset, float myFurnYOffset)
	;There is already a wood pile container located nearby, move the player there and use it instead.
	pPlayer.MoveTo(myWoodContEndRef)
	pPlayer.SetAngle(0.0, 0.0, myWoodContEndRef.GetAngleZ() + 180.0)
		
	;Move the marker back and then place the furniture object
	_DE_XMarker_WoodHarvestEndRef2.MoveTo(pPlayer, myFurnXOffSet, myFurnYOffset, -10.0)
	myWoodFurnEndRef = _DE_XMarker_WoodHarvestEndRef2.PlaceAtMe(_DE_WoodPilePlayer)
	myWoodFurnEndRef.SetScale(0.01)
	
	;Move the player 300 units away from current heading.
	pPlayer.MoveTo(pPlayer, myXOffset, myYOffset, 0.0)
	;place this near the player
	myWoodFurnStartRef = pPlayer.PlaceAtMe(_DE_WoodPilePlayer)
endFunction

function HarvestWoodWalking()
	;notification("Walking")
	_DE_WoodHarvestingAlias.ForceRefIfEmpty(pPlayer)
	Game.SetPlayerAIDriven(true)	
	pPlayer.EvaluatePackage()					;Player has wood, and walks to _DE_XMarker_WoodHarvestEndRef2, which is on top of myWoodFurnEndRef
	
	;Record the time the player starts walking
	pfThreadLastUpdateTime = GetCurrentGameTime()
	
	;Monitor for the package getting hung up
	;RegisterForSingleUpdate(1)
	
	;Delete the wood pickup furniture
	;myWoodFurnStartRef.MoveTo(myWoodFurnStartRef, 0.0, 0.0, 500.0)
	Wait(0.5)
	myWoodFurnStartRef.Disable()
	myWoodFurnStartRef.Delete()
	Wait(0.5)
	
	;fade up (during walk animation)
	_DE_Black.PopTo(_DE_FadeUp)
	
	Wait(5)
	if pPlayer.GetCurrentPackage() == _DE_PlayerHarvestWoodPathing2
		;notification("[Frostfall]Package got stuck, resolving...")
		HarvestWoodEnd()
	else
		;notification("[Frostfall]Package resolved successfully.")
	endif
	
endFunction

function HarvestWoodEnd()
	
	Game.SetPlayerAIDriven(false)
	
	;notification("Dropping wood")
	
	myWoodFurnEndRef.Activate(pPlayer)
	_DE_XMarker_WoodHarvestEndRef2.MoveTo(_DE_Anchor)
	Game.EnablePlayerControls()
	
	myHarvest = RandomInt(3, 7)
	_DE_WoodHarvestSuccess.Show(myHarvest)
	
	;Give the player back his firewood
	if myFirewoodCount > 0
		while Game.GetPlayer().GetItemCount(Firewood01) < myFireWoodCount
			pPlayer.AddItem(Firewood01, 1, true)		;silently				;one at a time is probably safer than all at once
		endwhile
	endif
	
	;Give the player the new firewood.
	myWoodContEndRef.AddItem(Firewood01, myHarvest)

	;notification("Done!")
endFunction

function UnregisterForAllEvents()
	RegisterForAnimationEvent(Game.GetPlayer(), "AddToInventory")
	RegisterForAnimationEvent(Game.GetPlayer(), "RemoveFromInventory")
endFunction
